# RAWREPAK - resort the raw 2D spectrum for SparsePak either
#  (a) in rows for a choice of 6 PA's, or
#  (b) radially
#
# The format of the sorting lists is a single column of fiber numbers
# in the first field. Additional comment fields are allowed, but no
# comment lines are allowed.
#
# This program uses only basic image manipulation in the noao.images package, and is
# currently hard-wired to extract the inner 4 pixels of each fiber spectrum
# (roughly the FWHM in the spatial dimension) for un-binned data.
#
# Modification History
# MAB Oct 22 2002 modified from repak.cl

procedure rawrepak (inmslst, flatlst, outmslst, x1, x2)

string inmslst		{ prompt = "Input image list" }
string flatlst		{ prompt = "Input flat list" }
string outmslst		{ prompt = "Output image list" }
real x1			{ prompt = "X coord. of fiber 82" }
real x2			{ prompt = "X coord. of fiber 1 (NB: x_82<x_1)" }

string sort="pa0"	{ prompt = "Sort as: pa0,pa30,pa60,pa90,pa120,pa150,rad",
			  enum="pa0|pa30|pa60|pa90|pa120|pa150|rad"}

real bias = 842.0	{ prompt = "Bias level; 0 for no subtraction" }

string pa0_file = "ifupkg$pa0.lst" 	{ prompt = "Name of PA = 0 sorting file" }
string pa30_file = "ifupkg$pa30.lst" 	{ prompt = "Name of PA = 30 sorting file" }
string pa60_file = "ifupkg$pa60.lst" 	{ prompt = "Name of PA = 60 sorting file" }
string pa90_file = "ifupkg$pa90.lst" 	{ prompt = "Name of PA = 90 sorting file" }
string pa120_file = "ifupkg$pa120.lst" 	{ prompt = "Name of PA = 120 sorting file" }
string pa150_file = "ifupkg$pa150.lst" 	{ prompt = "Name of PA = 150 sorting file" }
string rad_file = "ifupkg$rad.lst" 	{ prompt = "Name of radial sorting file" }

bool verbose = yes			{ prompt = "Report progress on image list" }
bool steps = yes			{ prompt = "Report progress on each image" }

struct *mylist1
struct *mylist2
struct *mylist3
struct *mylist4

begin

   int ii, ifib, ixs, ixe, xsize, ixso, ixeo, is, idx
   string iml, ims, oml, oms, sortlst, smaster, section, isect, osect
   string fml, fms, tempim, tempfim
   real xx1,xx2,dx,xx

# set up temp files

   sortlst = mktemp ( "ArOs" )  	
   iml =  mktemp ( "ApII" )  	
   fml =  mktemp ( "AomF" )  	
   oml =  mktemp ( "AomO" )  	
   tempim = mktemp ( "Ta0" )
   tempfim = mktemp ( "Ta0f" )
   delete ( sortlst, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( iml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( fml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( oml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   imdelete ( tempim, verify-, go_ahead+, default_action+,  >& "dev$null" )
   imdelete ( tempfim, verify-, go_ahead+, default_action+,  >& "dev$null" )

# get inputs

   sections ( inmslst, option="fullname", > iml )
   sections ( flatlst, option="fullname", > fml )
   sections ( outmslst, option="fullname", > oml )

   mylist1 = iml
   mylist2 = fml
   mylist3 = oml

   xx1 = x1
   xx2 = x2

# set image dimensions and steps

   is = 7

   dx = (xx2-xx1)/81.

   idx = int(dx-is+0.5)
   xsize = 82*idx
   section = "[1:"//xsize//",*]"

# set sorting file

   if ( sort=="pa0" ) smaster = pa0_file
   if ( sort=="pa30" ) smaster = pa30_file
   if ( sort=="pa60" ) smaster = pa60_file
   if ( sort=="pa90" ) smaster = pa90_file
   if ( sort=="pa120" ) smaster = pa120_file
   if ( sort=="pa150" ) smaster = pa150_file
   if ( sort=="rad" ) smaster = rad_file

   if ( verbose ) print ( "  > Reformatting SparsePak ms-file to ", sort )

# loop in images

   while ( fscan ( mylist1, ims ) != EOF ) {

	ii = fscan ( mylist2, fms )
	ii = fscan ( mylist3, oms )
	if ( verbose ) print ( "  > Reformatting ", ims, "  ->  ", oms )
	mylist4 = smaster

# copy image

	imarith ( ims//section, "*", "0.0", oms, verbose-,
                  calctype="real", pixtype="real", noact- )

# loop on fibers for each image

	ii = 1
        ixeo = 0
	while ( fscan ( mylist4, ifib ) != EOF ) {

	   xx = xx2 - real(ifib-1.)*dx
	   ixs = int(xx-real(idx)/2.+0.5)
	   ixe = ixs + idx - 1
	   ixso = 1 + ixeo
	   ixeo = ixso + (ixe-ixs)

	   isect = "["//ixs//":"//ixe//",*]"
	   osect = "["//ixso//":"//ixeo//",*]"
	
	   imarith ( ims//isect, "-", bias, tempim, verbose-,
		     calctype="real", pixtype="real", noact- )

	   imarith ( fms//isect, "-", bias, tempfim, verbose-,
		     calctype="real", pixtype="real", noact- )

	   imarith ( tempim, "/", tempfim, tempim, verbose-,
		     calctype="real", pixtype="real", noact- )

	   imcopy ( tempim, oms//osect, verbose- )
 	   if ( steps ) print( "    ", ims//isect, " -> ", oms//osect, "  : fiber ", 
  		  ifib, " -> position ", ii )
           imdelete ( tempim, verify-, go_ahead+, default_action+,  >& "dev$null" )
           imdelete ( tempfim, verify-, go_ahead+, default_action+,  >& "dev$null" )
	
 	   ii += 1

	}   

        delete ( sortlst, verify-, go_ahead+, allversions+, subfiles+,
	         >& "dev$null" )

   }

# clean up

   delete ( iml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( fml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( oml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   imdelete ( tempim, verify-, go_ahead+, default_action+,  >& "dev$null" )
   imdelete ( tempfim, verify-, go_ahead+, default_action+,  >& "dev$null" )

end

