# REPAK - sort the ms files for SparsePak either
#  (a) in rows for a choice of 6 PA's, or
#  (b) radially
#

# The format of the sorting lists is a single column of fiber numbers
# in the first field. Additional comment fields are allowed, but no
# comment lines are allowed.
#
# This program uses images.imutil.imstack
#
# Modification History
# DRA Mar 04 2002 initial hardwired script for PA = 0
# MAB Mar 05 2002 mod for image lists and choice of sorting, comment, etc.

procedure repak (inmslst, outmslst)

string inmslst		{ prompt = "Input ms file list" }
string outmslst		{ prompt = "Output ms file list" }

string sort="pa0"	{ prompt = "Sort as: pa0,pa30,pa60,pa90,pa120,pa150,rad",
			  enum="pa0|pa30|pa60|pa90|pa120|pa150|rad"}

string pa0_file = "ifupkg$pa0.lst" 	{ prompt = "Name of PA = 0 sorting file" }
string pa30_file = "ifupkg$pa30.lst" 	{ prompt = "Name of PA = 30 sorting file" }
string pa60_file = "ifupkg$pa60.lst" 	{ prompt = "Name of PA = 60 sorting file" }
string pa90_file = "ifupkg$pa90.lst" 	{ prompt = "Name of PA = 90 sorting file" }
string pa120_file = "ifupkg$pa120.lst" 	{ prompt = "Name of PA = 120 sorting file" }
string pa150_file = "ifupkg$pa150.lst" 	{ prompt = "Name of PA = 150 sorting file" }
string rad_file = "ifupkg$rad.lst" 	{ prompt = "Name of radial sorting file" }

bool verbose = yes	{ prompt = "Report progress" }

struct *mylist1
struct *mylist2
struct *mylist3

begin

   int ii, ifib
   string iml, ims, oml, oms, sortlst, smaster

# set up temp files

   sortlst = mktemp ( "RpKs" )  	
   iml =  mktemp ( "RpKi" )  	
   oml =  mktemp ( "RpKo" )  	
   delete ( sortlst, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( iml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( oml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )

# get input lists

   sections ( inmslst, option="fullname", > iml )
   sections ( outmslst, option="fullname", > oml )

   mylist1 = iml
   mylist2 = oml

# set sorting file

   if ( sort=="pa0" ) smaster = pa0_file
   if ( sort=="pa30" ) smaster = pa30_file
   if ( sort=="pa60" ) smaster = pa60_file
   if ( sort=="pa90" ) smaster = pa90_file
   if ( sort=="pa120" ) smaster = pa120_file
   if ( sort=="pa150" ) smaster = pa150_file
   if ( sort=="rad" ) smaster = rad_file

   if ( verbose ) print ( "  > Reformatting SparsePak ms-file to ", sort )

# loop in images

   while ( fscan ( mylist1, ims ) != EOF ) {

	ii = fscan ( mylist2, oms )
	if ( verbose ) print ( "  > Reformatting ", ims, "  ->  ", oms )
	mylist3 = smaster

# loop on fibers for each image

	ii = 1
	while ( fscan ( mylist3, ifib ) != EOF ) {
	   if ( ii==1 ) print( ims//"[*,"//ifib//"]", > sortlst)
	   else print( ims//"[*,"//ifib//"]", >> sortlst)
	   ii += 1
	}   

        imstack("@"//sortlst,oms)

        delete ( sortlst, verify-, go_ahead+, allversions+, subfiles+,
	         >& "dev$null" )

   }

# clean up

   delete ( iml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )
   delete ( oml, verify-, go_ahead+, allversions+, subfiles+,  >& "dev$null" )

end

